package com.programaker.api

import android.os.Build
import android.util.JsonReader
import android.util.Log
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import java.io.*
import java.lang.reflect.Type
import java.net.HttpURLConnection
import java.net.URL
import java.net.URLConnection
import javax.net.ssl.HttpsURLConnection


class ProgramakerApi(private val ApiRoot: String="https://programaker.com/api") {
    private val LogTag: String = "ProgramakerApi"
    private val TokenEntry: String = "ProgramakerAuthToken"

    // API
    fun Check(): Boolean {
        val conn = URL(getCheckUrl()).openConnection() as HttpURLConnection
        if (conn == null){
            Log.e(LogTag, "URL Connection not established, set to NULL")
            return false
        }
        try {
            addAuthHeader(conn)
        } catch(ex: TokenNotFoundException) {
            return false
        }

        val result: ProgramakerCheckResult
        try {
            result = parseJson(conn.inputStream, ProgramakerCheckResult::class.java)
        } catch(ex: JsonParseException) {
            ex.logError(LogTag)
            return false
        }
        return result.success
    }

    // Initialization
    init {
        // Disable connection reuse if necessary
        // HTTP connection reuse which was buggy pre-froyo
        if (Build.VERSION.SDK.toInt() < Build.VERSION_CODES.FROYO) {
            System.setProperty("http.keepAlive", "false")
        }
    }

    // Private functions
    private fun getCheckUrl(): String {
        return "$ApiRoot/v0/ping"
    }

    private fun addAuthHeader(conn: URLConnection) {
        // TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    private fun <T>parseJson(content: InputStream, resultClass: Type): T {
        val reader = InputStreamReader(content)
        val gson = Gson()
        return gson.fromJson(reader, resultClass)
    }
}

class JsonParseException(private val content: String, private val cls: Type) : Exception() {
    fun logError(tag: String) {
        Log.e(tag, "Cannot JSON parse: ${this.content} as ${this.cls}")
    }
}
